@@ -32,71 +32,51 @@
 #include "testDwaLookups.h"
 #include "testExistingStreams.h"
 #include "testFutureProofing.h"
+#include "testHeader.h"
 #include "testHuf.h"
-#include "testWav.h"
-#include "testChannels.h"
-#include "testAttributes.h"
-#include "testCustomAttributes.h"
-#include "testLineOrder.h"
-#include "testCompression.h"
-#include "testCopyPixels.h"
-#include "testRgba.h"
-#include "testRgbaThreading.h"
-#include "testLut.h"
-#include "testMagic.h"
-#include "testSampleImages.h"
-#include "testPreviewImage.h"
-#include "testConversion.h"
-#include "testStandardAttributes.h"
-#include "testNativeFormat.h"
-#include "testTiledRgba.h"
-#include "testTiledCompression.h"
-#include "testTiledCopyPixels.h"
-#include "testTiledLineOrder.h"
-#include "testScanLineApi.h"
-#include "testExistingStreams.h"
-#include "testYca.h"
-#include "testTiledYa.h"
+#include "testIDManifest.h"
+#include "testInputPart.h"
 #include "testIsComplete.h"
 #include "testLargeDataWindowOffsets.h"
-#include "testSharedFrameBuffer.h"
-#include "testMultiView.h"
+#include "testLineOrder.h"
+#include "testLut.h"
+#include "testMagic.h"
 #include "testMultiPartApi.h"
+#include "testMultiPartFileMixingBasic.h"
 #include "testMultiPartSharedAttributes.h"
 #include "testMultiPartThreading.h"
 #include "testMultiScanlinePartThreading.h"
 #include "testMultiTiledPartThreading.h"
-#include "testDeepScanLineBasic.h"
-#include "testCopyDeepScanLine.h"
-#include "testDeepScanLineMultipleRead.h"
-#include "testDeepScanLineHuge.h"
-#include "testDeepTiledBasic.h"
-#include "testCopyDeepTiled.h"
-#include "testCompositeDeepScanLine.h"
-#include "testMultiPartFileMixingBasic.h"
-#include "testInputPart.h"
-#include "testBackwardCompatibility.h"
-#include "testCopyMultiPartFile.h"
-#include "testPartHelper.h"
+#include "testMultiView.h"
+#include "testNativeFormat.h"
 #include "testOptimized.h"
 #include "testOptimizedInterleavePatterns.h"
-#include "testBadTypeAttributes.h"
-#include "testFutureProofing.h"
 #include "testPartHelper.h"
-#include "testDwaCompressorSimd.h"
+#include "testPreviewImage.h"
+#include "testRgba.h"
+#include "testRgbaThreading.h"
 #include "testRle.h"
-#include "testB44ExpLogTable.h"
-#include "testDwaLookups.h"
-#include "testIDManifest.h"
+#include "testSampleImages.h"
+#include "testScanLineApi.h"
+#include "testSharedFrameBuffer.h"
+#include "testStandardAttributes.h"
+#include "testTiledCompression.h"
+#include "testTiledCopyPixels.h"
+#include "testTiledLineOrder.h"
+#include "testTiledRgba.h"
+#include "testTiledYa.h"
+#include "testWav.h"
 #include "testXdr.h"
-#include "tmpDir.h"
+#include "testYca.h"
+
 #include "ImathRandom.h"
+#include "tmpDir.h"
 
 // system includes
 
 #include <errno.h>
-#include <stdlib.h>
 #include <iostream>
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 
@@ -112,24 +92,26 @@
 using namespace std;
 
 #define TEST_STRING(x) #x
-#define TEST(x,y)                                                       \
-    if (helpMode)\
-    {\
-	tests.insert(string(TEST_STRING(x)));\
-	suites.insert(string(y));\
-    }\
-    else if (argc < 2 || (!strcmp (argv[1], TEST_STRING(x)) || !strcmp (argv[1], y)))    \
-    {                                                                   \
-        cout << "\n=======\nRunning " << TEST_STRING(x) <<endl;                     \
-        x(tempDir);                                                     \
+#define TEST(x, y)                                                             \
+    if (helpMode)                                                              \
+    {                                                                          \
+        tests.insert (string (TEST_STRING (x)));                               \
+        suites.insert (string (y));                                            \
+    }                                                                          \
+    else if (                                                                  \
+        argc < 2 ||                                                            \
+        (!strcmp (argv[1], TEST_STRING (x)) || !strcmp (argv[1], y)))          \
+    {                                                                          \
+        cout << "\n=======\nRunning " << TEST_STRING (x) << endl;              \
+        x (tempDir);                                                           \
     }
 
-
-string makeTempDir()
+string
+makeTempDir ()
 {
     string tempDir;
 
-    IMATH_NAMESPACE::Rand48 rand48 (time ((time_t*)0) );
+    IMATH_NAMESPACE::Rand48 rand48 (time ((time_t*) 0));
     while (true)
     {
 #ifdef _WIN32
@@ -138,7 +120,7 @@ string makeTempDir()
         if (len == 0 || len > 4095)
         {
             cerr << "Cannot retrieve temporary directory" << endl;
-            exit(1);
+            exit (1);
         }
         tempDir = tmpbuf;
         // windows does this automatically
@@ -148,11 +130,11 @@ string makeTempDir()
         tempDir = IMF_TMP_DIR "OpenEXRTest_";
 #endif
         for (int i = 0; i < 8; ++i)
-            tempDir += ('A' + rand48.nexti() % 26);
+            tempDir += ('A' + rand48.nexti () % 26);
 
         std::cout << "tempDir = " << tempDir << std::endl;
 
-        int status = mkdir(tempDir.c_str(), 0777);
+        int status = mkdir (tempDir.c_str (), 0777);
 
         if (status == 0)
         {
@@ -162,18 +144,19 @@ string makeTempDir()
 
         if (errno != EEXIST)
         {
-            std::cerr << "ERROR -- mkdir(" << tempDir << ") failed: "
-                         "errno = " << errno << std::endl;
-            exit(1);
+            std::cerr << "ERROR -- mkdir(" << tempDir
+                      << ") failed: "
+                         "errno = "
+                      << errno << std::endl;
+            exit (1);
         }
     }
 
     return tempDir;
-
 }
 
 int
-main (int argc, char *argv[])
+main (int argc, char* argv[])
 {
     // Create temporary files in a uniquely named private temporary
     // subdirectory of IMF_TMP_DIR to avoid colliding with other
@@ -182,18 +165,15 @@ main (int argc, char *argv[])
     std::string tempDir;
 
     bool helpMode = false;
-    if( argc==2 && (strcmp(argv[1],"--help")==0 || strcmp(argv[1],"-h")==0))
+    if (argc == 2 &&
+        (strcmp (argv[1], "--help") == 0 || strcmp (argv[1], "-h") == 0))
     {
-            helpMode = true;
+        helpMode = true;
     }
     set<string> tests;
     set<string> suites;
 
-
-    if ( !helpMode )
-    {
-	    tempDir = makeTempDir();
-    }
+    if (!helpMode) { tempDir = makeTempDir (); }
 
     // NB: If you add a test here, make sure to enumerate it in the
     // CMakeLists.txt so it runs as part of the test suite
@@ -253,54 +233,57 @@ main (int argc, char *argv[])
     TEST (testDwaLookups, "core");
     TEST (testIDManifest, "core");
     TEST (testCpuId, "core");
+    TEST (testHeader, "basic");
 
     // NB: If you add a test here, make sure to enumerate it in the
     // CMakeLists.txt so it runs as part of the test suite
-    
+
     //#ifdef ENABLE_IMFHUGETEST
     // defined via configure with --enable-imfhugetest=yes/no
 #if 0
         TEST (testDeepScanLineHuge, "deep");
-#endif    
+#endif
 
-
-    if ( helpMode )
+    if (helpMode)
     {
         cout << "OpenEXRTest runs a series of tests to confirm\n"
-		"correct behavior of the OpenEXR library.\n"
-	        "If all is correct, OpenEXRTest will complete without\n"
-		"crashing or leaking memory.\n";
-	cout << "\n";
-	cout << "If a test fails, an individual test can be re-run, avoiding\n"
-		"the wait for previous tests to complete. This allows easier debugging\n"
-                "of the failure.\n";
-	cout << "\n";
-	cout << "A 'suite' of tests can also be run, to allow a subset of\n"
-	     << "tests to run. This is useful as an initial confirmation\n"
-	     << "that a modification to the library has not introduced an error.\n"
-	     << "Suites can be run in parallel for speed. Every test is in one suite.\n";
-	cout << "\n";
-	cout << "usage:\n"
-	     << "OpenEXRTest           : with no arguments, run all tests\n"
-	     << "OpenEXRTest TEST      : run only specific test, then quit\n"
-             << "OpenEXRTest SUITE     : run all the tests in the given SUITE\n";
-	cout << "\n";
-	cout << "available TESTs:\n";
-	for ( auto i = tests.begin() ; i!= tests.end() ; ++i)
-	{
-		cout << ' ' << *i << endl;
-	}
-	cout << "\n";
-	cout << "available SUITEs:\n";
-	for ( auto i = suites.begin() ; i!= suites.end() ; ++i )
-	{
-		cout << ' ' << *i << endl;
-	}
-    } 
+                "correct behavior of the OpenEXR library.\n"
+                "If all is correct, OpenEXRTest will complete without\n"
+                "crashing or leaking memory.\n";
+        cout << "\n";
+        cout
+            << "If a test fails, an individual test can be re-run, avoiding\n"
+               "the wait for previous tests to complete. This allows easier debugging\n"
+               "of the failure.\n";
+        cout << "\n";
+        cout
+            << "A 'suite' of tests can also be run, to allow a subset of\n"
+            << "tests to run. This is useful as an initial confirmation\n"
+            << "that a modification to the library has not introduced an error.\n"
+            << "Suites can be run in parallel for speed. Every test is in one suite.\n";
+        cout << "\n";
+        cout
+            << "usage:\n"
+            << "OpenEXRTest           : with no arguments, run all tests\n"
+            << "OpenEXRTest TEST      : run only specific test, then quit\n"
+            << "OpenEXRTest SUITE     : run all the tests in the given SUITE\n";
+        cout << "\n";
+        cout << "available TESTs:\n";
+        for (auto i = tests.begin (); i != tests.end (); ++i)
+        {
+            cout << ' ' << *i << endl;
+        }
+        cout << "\n";
+        cout << "available SUITEs:\n";
+        for (auto i = suites.begin (); i != suites.end (); ++i)
+        {
+            cout << ' ' << *i << endl;
+        }
+    }
     else
     {
         cout << "removing temp dir " << tempDir << endl;
-        rmdir (tempDir.c_str());
+        rmdir (tempDir.c_str ());
 
 #ifdef OPENEXR_IMF_HAVE_LINUX_PROCFS
 
@@ -311,16 +294,13 @@ main (int argc, char *argv[])
         cout << "open file descriptors:" << endl;
 
         stringstream ss;
-        ss << "ls -lG /proc/" << getpid() << "/fd";
-            
-        if (system (ss.str().c_str()) == -1)
-        {
-            cout << "failed to run ls\n";
-        }
+        ss << "ls -lG /proc/" << getpid () << "/fd";
+
+        if (system (ss.str ().c_str ()) == -1) { cout << "failed to run ls\n"; }
 
         cout << endl;
 
 #endif
-   }
+    }
     return 0;
 }
